package com.xkrs.dao;

import com.xkrs.model.entity.CityEntity;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @author xkrs
 */
@Component
public interface CityDao extends JpaRepository<CityEntity,Long>, JpaSpecificationExecutor<CityEntity> {

    /**
     * 根据省编码查询城市有关信息
     * @param proCode
     * @return
     */
    @Query(value = "select city_name,city_code,city_latitude,city_longitude from city where pro_code = ?",nativeQuery = true)
    List<Object[]> selectCityNameAndLatAndLog(String proCode);

    /**
     * 查询所有市和对应的编码
     * @param code
     * @return
     */
    @Query(value = "select city_name cityname,city_code citycode from city where pro_code = :code",nativeQuery = true)
    List<Map<String,Object>> selectCityName(String code);

    /**
     * 根据市编码查询信息
     * @param cityCode
     * @return
     */
    CityEntity findByCityCode(String cityCode);

    /**
     * 根据省编码查询信息
     * @param proCode
     * @return
     */
    List<CityEntity> findByProCode(String proCode);

    /**
     * 根据市名称查询信息
     * @param cityName
     * @return
     */
    //List<CityEntity> findByCityName(String cityName);

    /**
     * 根据省名称查询信息
     * @param proName
     * @return
     */
    List<CityEntity> findByProName(String proName);

    /**
     * 根据市名查询信息
     * @param cityName
     * @return
     */
    CityEntity findByCityName(String cityName);
}
