package com.xkrs.utils;

import com.xkrs.dao.FirePointDao;
import com.xkrs.model.entity.FirePointEntity;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author XinYi Song
 */

@Component
public class Query {

    @Resource
    private FirePointDao firePointDao;

    /**
     * 动态多条件查询项目信息(省)
     * @param
     * @return
     */
    public List<FirePointEntity> selectFirePointByPro(String address, String cityCode, String satelliteType, String landType, String startTime, String endTime) {
        Specification<FirePointEntity> specification = new Specification<FirePointEntity>() {
            @Override
            public Predicate toPredicate(Root<FirePointEntity> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if(address != null && !"".equals(address)){
                    list.add(criteriaBuilder.like(root.get("firePointAddress").as(String.class), "%" + address + "%"));
                }
                if (cityCode != null && !"".equals(cityCode)) {
                    list.add(criteriaBuilder.like(root.get("countyCode").as(String.class), "%" + cityCode + "%"));
                }
                if (satelliteType != null && !"".equals(satelliteType)) {
                    list.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), satelliteType));
                }
                if (landType != null && !"".equals(landType)) {
//                    if(address.equals("山东省")){
//                        list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                    }
                    list.add(criteriaBuilder.equal(root.get("landType").as(String.class), landType));
                }
//                else {
//                    if(address.equals("山东省")){
//                        list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                    }
//                }
                if(startTime != null && !"".equals(startTime)){
                    list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
                }
                LocalDateTime localDateTime = LocalDateTime.now();
                LocalDateTime localDateTime2 = localDateTime.plusDays(1);
                String endTimes = DateTimeUtil.dateTimeToString(localDateTime2);
                // 当前时间减30
                LocalDateTime localDateTime1 = localDateTime.minusDays(30);
                String startTimes = DateTimeUtil.dateTimeToString(localDateTime1);
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimes));
                if(endTime != null && !"".equals(endTime)){
                    list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
                }
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimes));
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 动态多条件查询项目信息(市)
     * @param
     * @return
     */
    public List<FirePointEntity> selectFirePointByCity(String proName, String address, String cityCode,String satelliteType,String landType,String startTime,String endTime) {
        Specification<FirePointEntity> specification = new Specification<FirePointEntity>() {
            @Override
            public Predicate toPredicate(Root<FirePointEntity> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if(address != null && !"".equals(address)){
                    list.add(criteriaBuilder.like(root.get("firePointAddress").as(String.class), "%" + address + "%"));
                }
                if (cityCode != null && !"".equals(cityCode)) {
                    list.add(criteriaBuilder.equal(root.get("countyCode").as(String.class),cityCode));
                }
                if (satelliteType != null && !"".equals(satelliteType)) {
                    list.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), satelliteType));
                }
                if (landType != null && !"".equals(landType)) {
//                    if(proName.equals("山东省")){
//                        list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                    }
                    list.add(criteriaBuilder.equal(root.get("landType").as(String.class), landType));
                }
//                else {
//                    if(proName.equals("山东省")){
//                        list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                    }
//                }
                if(startTime != null && !"".equals(startTime)){
                    list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
                }
                LocalDateTime localDateTime = LocalDateTime.now();
                LocalDateTime localDateTime2 = localDateTime.plusDays(1);
                String endTimes = DateTimeUtil.dateTimeToString(localDateTime2);
                // 当前时间减30
                LocalDateTime localDateTime1 = localDateTime.minusDays(30);
                String startTimes = DateTimeUtil.dateTimeToString(localDateTime1);
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimes));
                if(endTime != null && !"".equals(endTime)){
                    list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
                }
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimes));
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 动态多条件查询项目信息(县)
     * @param
     * @return
     */
    public List<FirePointEntity> selectFirePointByCounty(String proName, String cityCode,String satelliteType,String landType,String startTime,String endTime) {
        Specification<FirePointEntity> specification = new Specification<FirePointEntity>() {
            @Override
            public Predicate toPredicate(Root<FirePointEntity> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if (cityCode != null && !"".equals(cityCode)) {
                    list.add(criteriaBuilder.equal(root.get("countyCode").as(String.class),cityCode));
                }
                if (satelliteType != null && !"".equals(satelliteType)) {
                    list.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), satelliteType));
                }
                if (landType != null && !"".equals(landType)) {
//                    if(proName.equals("山东省")){
//                        list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                    }
                    list.add(criteriaBuilder.equal(root.get("landType").as(String.class), landType));
                }
//                else {
//                    if(proName.equals("山东省")){
//                        list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                    }
//                }
                if(startTime != null && !"".equals(startTime)){
                    list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
                }
                LocalDateTime localDateTime = LocalDateTime.now();
                LocalDateTime localDateTime2 = localDateTime.plusDays(1);
                String endTimes = DateTimeUtil.dateTimeToString(localDateTime2);
                // 当前时间减30
                LocalDateTime localDateTime1 = localDateTime.minusDays(30);
                String startTimes = DateTimeUtil.dateTimeToString(localDateTime1);
                list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTimes));
                if(endTime != null && !"".equals(endTime)){
                    list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
                }
                list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTimes));
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一周的火点
     * @param startTime
     * @param endTime
     * @return
     */
    public List<FirePointEntity> selectFirePointBetweenThree(String startTime,String endTime) {
        Specification<FirePointEntity> specification = new Specification<FirePointEntity>() {
            @Override
            public Predicate toPredicate(Root<FirePointEntity> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if(startTime != null && !"".equals(startTime)){
                    list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
                }
                if(endTime != null && !"".equals(endTime)){
                    list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
                }
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一个月的火点
     * @param startTime
     * @param endTime
     * @return
     */
    public List<FirePointEntity> selectFirePointByMonth(String address,String startTime,String endTime) {
        Specification<FirePointEntity> specification = new Specification<FirePointEntity>() {
            @Override
            public Predicate toPredicate(Root<FirePointEntity> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if(address != null && !"".equals(address)){
                    list.add(criteriaBuilder.like(root.get("firePointAddress").as(String.class), "%" + address + "%"));
                }
                if(startTime != null && !"".equals(startTime)){
                    list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
                }
                if(endTime != null && !"".equals(endTime)){
                    list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
                }
//                if(address.equals("山东省")){
//                    list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                }
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一个月的火点信息（市）
     * @param address
     * @param startTime
     * @param endTime
     * @return
     */
    public List<FirePointEntity> selectFirePointByMonthCity(String proName, String address,String startTime,String endTime) {
        Specification<FirePointEntity> specification = new Specification<FirePointEntity>() {
            @Override
            public Predicate toPredicate(Root<FirePointEntity> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if(address != null && !"".equals(address)){
                    list.add(criteriaBuilder.like(root.get("firePointAddress").as(String.class), "%" + address + "%"));
                }
                if(startTime != null && !"".equals(startTime)){
                    list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
                }
                if(endTime != null && !"".equals(endTime)){
                    list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
                }
//                if(proName.equals("山东省")){
//                    list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                }
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return firePointDao.findAll(specification);
    }

    /**
     * 查询近一个月的火点信息（县）
     * @param countyCode
     * @param startTime
     * @param endTime
     * @return
     */
    public List<FirePointEntity> selectFirePointByMonthCounty(String proName, String countyCode,String startTime,String endTime) {
        Specification<FirePointEntity> specification = new Specification<FirePointEntity>() {
            @Override
            public Predicate toPredicate(Root<FirePointEntity> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> list = new ArrayList<>();
                if(countyCode != null && !"".equals(countyCode)){
                    list.add(criteriaBuilder.like(root.get("countyCode").as(String.class), countyCode));
                }
                if(startTime != null && !"".equals(startTime)){
                    list.add(criteriaBuilder.greaterThanOrEqualTo(root.get("satelliteTime").as(String.class), startTime));
                }
                if(endTime != null && !"".equals(endTime)){
                    list.add(criteriaBuilder.lessThanOrEqualTo(root.get("satelliteTime").as(String.class), endTime));
                }
//                if(proName.equals("山东省")){
//                    list.add(criteriaBuilder.notEqual(root.get("landType").as(String.class), "耕地"));
//                }
                Predicate[] predicates = new Predicate[list.size()];
                return criteriaBuilder.and(list.toArray(predicates));
            }
        };
        return firePointDao.findAll(specification);
    }


}